<?php
namespace DDTrace {
use ArrayIterator;
use DDTrace\Contracts\SpanContext as SpanContextInterface;
use DDTrace\Data\SpanContext as SpanContextData;
final class SpanContext extends SpanContextData
{
    public function __construct($traceId, $spanId, $parentId = null, array $baggageItems = array(), $isDistributedTracingActivationContext = false)
    {
        $this->traceId = $traceId;
        $this->spanId = $spanId;
        $this->parentId = $parentId ?: null;
        $this->baggageItems = $baggageItems;
        $this->isDistributedTracingActivationContext = $isDistributedTracingActivationContext;
    }
    public static function createAsChild(SpanContextInterface $parentContext, $startTime = null)
    {
        $origin = \property_exists($parentContext, 'origin') ? $parentContext->origin : null;
        if (!$parentContext->isDistributedTracingActivationContext() || !active_span()) {
            if ($parentContext->isDistributedTracingActivationContext()) {
                set_distributed_tracing_context($parentContext->getTraceId(), $parentContext->getSpanId(), $origin);
            }
            if ($startTime) {
                start_span($startTime);
            } else {
                start_span();
            }
        }
        $instance = new self($parentContext->getTraceId(), active_span()->id, $parentContext->getSpanId(), $parentContext->getAllBaggageItems(), false);
        $instance->parentContext = $parentContext;
        $instance->setPropagatedPrioritySampling($parentContext->getPropagatedPrioritySampling());
        if ($origin) {
            $instance->origin = $origin;
        }
        return $instance;
    }
    public static function createAsRoot(array $baggageItems = array(), $startTime = null)
    {
        if (!active_span()) {
            if ($startTime) {
                start_span($startTime);
            } else {
                start_span();
            }
        }
        $nextId = active_span()->id;
        return new self($nextId, $nextId, null, $baggageItems, false);
    }
    public function getTraceId()
    {
        return $this->traceId;
    }
    public function getSpanId()
    {
        return $this->spanId;
    }
    public function getParentId()
    {
        return $this->parentId;
    }
    #[\ReturnTypeWillChange]
    public function getIterator()
    {
        return new ArrayIterator($this->baggageItems);
    }
    public function getPropagatedPrioritySampling()
    {
        return $this->propagatedPrioritySampling;
    }
    public function setPropagatedPrioritySampling($propagatedPrioritySampling)
    {
        $this->propagatedPrioritySampling = $propagatedPrioritySampling;
    }
    public function getBaggageItem($key)
    {
        return array_key_exists($key, $this->baggageItems) ? $this->baggageItems[$key] : null;
    }
    public function withBaggageItem($key, $value)
    {
        return new self($this->traceId, $this->spanId, $this->parentId, array_merge($this->baggageItems, array($key => $value)));
    }
    public function getAllBaggageItems()
    {
        return $this->baggageItems;
    }
    public function isEqual(SpanContextInterface $spanContext)
    {
        if ($spanContext instanceof SpanContextData) {
            return $this->traceId === $spanContext->traceId && $this->spanId === $spanContext->spanId && $this->parentId === $spanContext->parentId && $this->baggageItems === $spanContext->baggageItems;
        }
        return $this->traceId === $spanContext->getTraceId() && $this->spanId === $spanContext->getSpanId() && $this->parentId === $spanContext->getParentId() && $this->baggageItems === $spanContext->getAllBaggageItems();
    }
    public function isDistributedTracingActivationContext()
    {
        return $this->isDistributedTracingActivationContext;
    }
    public function isHostRoot()
    {
        return $this->parentContext === null || $this->parentContext->isDistributedTracingActivationContext();
    }
}
}
namespace DDTrace {
use DDTrace\Data\Span as DataSpan;
use DDTrace\Exceptions\InvalidSpanArgument;
use DDTrace\Log\LoggingTrait;
use DDTrace\SpanContext;
use DDTrace\Http\Urls;
use DDTrace\Processing\TraceAnalyticsProcessor;
use Exception;
use InvalidArgumentException;
use Throwable;
class Span extends DataSpan
{
    use LoggingTrait;
    private static $metricNames = array(Tag::ANALYTICS_KEY => true);
    private static $specialTags = array(Tag::ANALYTICS_KEY => true, Tag::ERROR => true, Tag::ERROR_MSG => true, Tag::SERVICE_NAME => true, Tag::RESOURCE_NAME => true, Tag::SPAN_TYPE => true, Tag::HTTP_URL => true, Tag::HTTP_STATUS_CODE => true, Tag::MANUAL_KEEP => true, Tag::MANUAL_DROP => true, Tag::SERVICE_VERSION => true);
    public $internalSpan;
    public function __construct(SpanData $internalSpan, SpanContext $context)
    {
        $this->internalSpan = $internalSpan;
        $this->context = $context;
    }
    public function getTraceId()
    {
        return $this->context->traceId;
    }
    public function getSpanId()
    {
        return $this->context->spanId;
    }
    public function getParentId()
    {
        return $this->context->parentId;
    }
    public function overwriteOperationName($operationName)
    {
        $this->internalSpan->name = $operationName;
    }
    public function getResource()
    {
        return $this->internalSpan->resource;
    }
    public function getService()
    {
        return $this->internalSpan->service;
    }
    public function getType()
    {
        return $this->internalSpan->type;
    }
    public function getStartTime()
    {
        return (int) ($this->startTime / 1000);
    }
    public function getDuration()
    {
        return (int) ($this->duration / 1000);
    }
    public function setTag($key, $value, $setIfFinished = false)
    {
        if ($this->isFinished() && !$setIfFinished) {
            return;
        }
        if ($key !== (string) $key) {
            throw InvalidSpanArgument::forTagKey($key);
        }
        if (is_object($value) && $key !== Tag::ERROR) {
            return;
        }
        if (array_key_exists($key, self::$specialTags)) {
            if ($key === Tag::ERROR) {
                $this->setError($value);
                return;
            }
            if ($key === Tag::ERROR_MSG) {
                $this->internalSpan->meta[$key] = (string) $value;
                $this->setError(true);
                return;
            }
            if ($key === Tag::SERVICE_NAME) {
                $this->internalSpan->service = $value;
                return;
            }
            if ($key === Tag::MANUAL_KEEP) {
                GlobalTracer::get()->setPrioritySampling(Sampling\PrioritySampling::USER_KEEP);
                return;
            }
            if ($key === Tag::MANUAL_DROP) {
                GlobalTracer::get()->setPrioritySampling(Sampling\PrioritySampling::USER_REJECT);
                return;
            }
            if ($key === Tag::RESOURCE_NAME) {
                $this->internalSpan->resource = (string) $value;
                return;
            }
            if ($key === Tag::SPAN_TYPE) {
                $this->internalSpan->type = $value;
                return;
            }
            if ($key === Tag::HTTP_URL) {
                $value = Urls::sanitize((string) $value);
            }
            if ($key === Tag::HTTP_STATUS_CODE && $value >= 500) {
                $this->hasError = true;
                if (!isset($this->internalSpan->meta[Tag::ERROR_TYPE])) {
                    $this->internalSpan->meta[Tag::ERROR_TYPE] = 'Internal Server Error';
                }
            }
            if ($key === Tag::SERVICE_VERSION) {
                $this->setTag(Tag::VERSION, $value);
            }
            if (array_key_exists($key, self::$metricNames)) {
                $this->setMetric($key, $value);
                return;
            }
        }
        $this->internalSpan->meta[$key] = (string) $value;
    }
    public function getTag($key)
    {
        if (isset($this->internalSpan->meta) && array_key_exists($key, $this->internalSpan->meta)) {
            return $this->internalSpan->meta[$key];
        }
        return null;
    }
    public function getAllTags()
    {
        return isset($this->internalSpan->meta) ? $this->internalSpan->meta : array();
    }
    public function hasTag($name)
    {
        return array_key_exists($name, $this->getAllTags());
    }
    public function setMetric($key, $value)
    {
        if ($key === Tag::ANALYTICS_KEY) {
            TraceAnalyticsProcessor::normalizeAnalyticsValue($this->internalSpan->metrics, $value);
            return;
        }
        $this->internalSpan->metrics[$key] = $value;
    }
    public function getMetrics()
    {
        return isset($this->internalSpan->metrics) ? $this->internalSpan->metrics : array();
    }
    public function setResource($resource)
    {
        $this->internalSpan->resource = (string) $resource;
    }
    public function setError($error)
    {
        if ($error instanceof Exception || $error instanceof Throwable) {
            $this->hasError = true;
            $this->internalSpan->meta[Tag::ERROR_MSG] = $error->getMessage();
            $this->internalSpan->meta[Tag::ERROR_TYPE] = get_class($error);
            $this->internalSpan->meta[Tag::ERROR_STACK] = $error->getTraceAsString();
            return;
        }
        if (is_bool($error)) {
            $this->hasError = $error;
            return;
        }
        if (is_null($error)) {
            $this->hasError = false;
        }
        throw InvalidSpanArgument::forError($error);
    }
    public function setRawError($message, $type)
    {
        $this->hasError = true;
        $this->internalSpan->meta[Tag::ERROR_MSG] = $message;
        $this->internalSpan->meta[Tag::ERROR_TYPE] = $type;
    }
    public function hasError()
    {
        return $this->hasError;
    }
    public function finish($finishTime = null)
    {
        if (!$this->isFinished()) {
            $current_span = active_span();
            if ($current_span != $this->internalSpan) {
                if (close_spans_until($this->internalSpan)) {
                    self::logWarning('Closed multiple spans at once while finishing a span named \'{name}\'. ' . 'The topmost span was named \'{topname}\'.', array('name' => $this->getOperationName(), 'topname' => $current_span->name));
                } else {
                    self::logDebug('Found unexpected span with name \'{name}\' on top of the span stack.', array('name' => $current_span->name));
                }
            }
            close_span($finishTime ?: 0);
        }
    }
    public function isFinished()
    {
        return $this->duration !== 0;
    }
    public function getOperationName()
    {
        return $this->internalSpan->name;
    }
    public function getContext()
    {
        return $this->context;
    }
    public function log(array $fields = array(), $timestamp = null)
    {
        foreach ($fields as $key => $value) {
            if ($key === Tag::LOG_EVENT && $value === Tag::ERROR) {
                $this->setError(true);
            } elseif ($key === Tag::LOG_ERROR || $key === Tag::LOG_ERROR_OBJECT) {
                $this->setError($value);
            } elseif ($key === Tag::LOG_MESSAGE) {
                $this->internalSpan->meta[Tag::ERROR_MSG] = (string) $value;
            } elseif ($key === Tag::LOG_STACK) {
                $this->setTag(Tag::ERROR_STACK, $value);
            }
        }
    }
    public function addBaggageItem($key, $value)
    {
        $this->context = $this->context->withBaggageItem($key, $value);
    }
    public function getBaggageItem($key)
    {
        return $this->context->getBaggageItem($key);
    }
    public function getAllBaggageItems()
    {
        return $this->context->baggageItems;
    }
    public function setTraceAnalyticsCandidate($value = true)
    {
        return $this;
    }
    public function isTraceAnalyticsCandidate()
    {
        return false;
    }
    public function __destruct()
    {
        if (!$this->isFinished()) {
            self::logWarning('The span with name \'{name}\' was closed without being finished first.', array('name' => $this->getOperationName()));
        }
    }
}
}
namespace DDTrace {
use DDTrace\Contracts\Tracer as TracerInterface;
class StartSpanOptionsFactory
{
    public static function createForWebRequest(TracerInterface $tracer, array $options = array(), array $headers = array())
    {
        if (\ddtrace_config_distributed_tracing_enabled() && ($spanContext = $tracer->extract(Format::HTTP_HEADERS, $headers))) {
            $options[Reference::CHILD_OF] = $spanContext;
        }
        return StartSpanOptions::create($options);
    }
}
}
namespace DDTrace {
class Time
{
    public static function now()
    {
        return (int) (microtime(true) * 1000 * 1000);
    }
    public static function fromMicrotime($microtime)
    {
        return (int) $microtime * 1000 * 1000;
    }
    public static function isValid($time)
    {
        return $time === (int) $time && ctype_digit((string) $time) && strlen((string) $time) === 16;
    }
}
}
namespace DDTrace\Transport {
use DDTrace\Contracts\Tracer;
use DDTrace\Encoder;
use DDTrace\GlobalTracer;
use DDTrace\Log\LoggingTrait;
use DDTrace\Sampling\PrioritySampling;
use DDTrace\Transport;
final class Http implements Transport
{
    use LoggingTrait;
    const AGENT_HOST_ENV = 'DD_AGENT_HOST';
    const TRACE_AGENT_URL_ENV = 'DD_TRACE_AGENT_URL';
    const TRACE_AGENT_PORT_ENV = 'DD_TRACE_AGENT_PORT';
    const AGENT_TIMEOUT_ENV = 'DD_TRACE_AGENT_TIMEOUT';
    const AGENT_CONNECT_TIMEOUT_ENV = 'DD_TRACE_AGENT_CONNECT_TIMEOUT';
    const DEFAULT_AGENT_HOST = 'localhost';
    const DEFAULT_TRACE_AGENT_PORT = '8126';
    const PRIORITY_SAMPLING_TRACE_AGENT_PATH = '/v0.4/traces';
    const DEFAULT_AGENT_CONNECT_TIMEOUT = 100;
    const DEFAULT_AGENT_TIMEOUT = 500;
    private $encoder;
    private $headers = array();
    private $config;
    public function __construct(Encoder $encoder, array $config = array())
    {
        $this->configure($config);
        $this->encoder = $encoder;
        $this->setHeader('Datadog-Meta-Lang', 'php');
        $this->setHeader('Datadog-Meta-Lang-Version', \PHP_VERSION);
        $this->setHeader('Datadog-Meta-Lang-Interpreter', \PHP_SAPI);
        $this->setHeader('Datadog-Meta-Tracer-Version', DD_TRACE_VERSION);
        $containerId = \DDTrace\System\container_id();
        if ($containerId) {
            $this->setHeader('Datadog-Container-Id', $containerId);
        }
    }
    private function configure($config)
    {
        $host = ddtrace_config_read_env_or_ini(self::AGENT_HOST_ENV) ?: self::DEFAULT_AGENT_HOST;
        $port = ddtrace_config_read_env_or_ini(self::TRACE_AGENT_PORT_ENV) ?: self::DEFAULT_TRACE_AGENT_PORT;
        $traceAgentUrl = ddtrace_config_read_env_or_ini(self::TRACE_AGENT_URL_ENV) ?: "http://{$host}:{$port}";
        $path = self::PRIORITY_SAMPLING_TRACE_AGENT_PATH;
        $endpoint = "{$traceAgentUrl}{$path}";
        $this->config = array_merge(array('endpoint' => $endpoint, 'connect_timeout' => ddtrace_config_read_env_or_ini(self::AGENT_CONNECT_TIMEOUT_ENV) ?: self::DEFAULT_AGENT_CONNECT_TIMEOUT, 'timeout' => ddtrace_config_read_env_or_ini(self::AGENT_TIMEOUT_ENV) ?: self::DEFAULT_AGENT_TIMEOUT), $config);
    }
    public function send(Tracer $tracer)
    {
        $tracesCount = $tracer->getTracesCount();
        $tracesPayload = $this->encoder->encodeTraces($tracer);
        if ($tracesCount === 0) {
            self::logDebug('No finished traces to be sent to the agent');
        }
        self::logDebug('About to send trace(s) to the agent');
        $this->sendRequest($this->config['endpoint'], $this->headers, $tracesPayload, $tracesCount);
    }
    public function setHeader($key, $value)
    {
        $this->headers[(string) $key] = (string) $value;
    }
    public function getConfig()
    {
        return $this->config;
    }
    private function sendRequest($url, array $headers, $body, $tracesCount)
    {
        $bodySize = strlen($body);
        if ($bodySize > \dd_trace_env_config('DD_TRACE_AGENT_MAX_PAYLOAD_SIZE')) {
            self::logError('Agent request payload of {bytes} bytes exceeds configured limit; dropping request', array('bytes' => $bodySize));
            return;
        }
        $curlHeaders = array();
        if (!isset($headers['Expect'])) {
            $curlHeaders[] = 'Expect:';
        }
        foreach ($headers as $key => $value) {
            $curlHeaders[] = "{$key}: {$value}";
        }
        $bgsEnabled = \dd_trace_env_config('DD_TRACE_BGS_ENABLED') && \dd_trace_env_config('DD_TRACE_BETA_SEND_TRACES_VIA_THREAD');
        if ($bgsEnabled && $this->encoder->getContentType() === 'application/msgpack' && \dd_trace_send_traces_via_thread($tracesCount, $curlHeaders, $body)) {
            return;
        }
        if ($this->isLogDebugActive() && function_exists('dd_tracer_circuit_breaker_info')) {
            self::logDebug('circuit breaker status: closed => {closed}, total_failures => {total_failures},' . 'consecutive_failures => {consecutive_failures}, opened_timestamp => {opened_timestamp}, ' . 'last_failure_timestamp=> {last_failure_timestamp}', dd_tracer_circuit_breaker_info());
        }
        if (function_exists('dd_tracer_circuit_breaker_can_try') && !dd_tracer_circuit_breaker_can_try()) {
            self::logError('Reporting of spans skipped due to open circuit breaker');
            return;
        }
        $curlHeaders = \array_merge($curlHeaders, array('Content-Type: ' . $this->encoder->getContentType(), 'X-Datadog-Trace-Count: ' . $tracesCount));
        $handle = curl_init($url);
        curl_setopt($handle, CURLOPT_POST, true);
        curl_setopt($handle, CURLOPT_POSTFIELDS, $body);
        curl_setopt($handle, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($handle, CURLOPT_TIMEOUT_MS, $this->config['timeout']);
        curl_setopt($handle, CURLOPT_CONNECTTIMEOUT_MS, $this->config['connect_timeout']);
        $isDebugEnabled = \ddtrace_config_debug_enabled();
        if ($isDebugEnabled) {
            $verbose = \fopen('php://temp', 'w+b');
            \curl_setopt($handle, \CURLOPT_VERBOSE, true);
            \curl_setopt($handle, \CURLOPT_STDERR, $verbose);
        }
        curl_setopt($handle, CURLOPT_HTTPHEADER, $curlHeaders);
        if (($response = curl_exec($handle)) === false) {
            $curlTimedout = \version_compare(\PHP_VERSION, '5.5', '<') ? \CURLE_OPERATION_TIMEOUTED : \CURLE_OPERATION_TIMEDOUT;
            $errno = \curl_errno($handle);
            $extra = '';
            if ($errno === $curlTimedout) {
                $timeout = $this->config['timeout'];
                $connectTimeout = $this->config['connect_timeout'];
                $extra = " (TIMEOUT_MS={$timeout}, CONNECTTIMEOUT_MS={$connectTimeout})";
            }
            self::logError('Reporting of spans failed: {num} / {error}{extra}', array('error' => curl_error($handle), 'num' => $errno, 'extra' => $extra));
            if ($isDebugEnabled && $verbose !== false) {
                @\rewind($verbose);
                $verboseLog = @\stream_get_contents($verbose);
                if ($verboseLog !== false) {
                    self::logError($verboseLog);
                } else {
                    self::logError('Error while retrieving curl error log');
                }
            }
            function_exists('dd_tracer_circuit_breaker_register_error') && dd_tracer_circuit_breaker_register_error();
            return;
        }
        $statusCode = curl_getinfo($handle, CURLINFO_HTTP_CODE);
        curl_close($handle);
        if ($statusCode === 415) {
            self::logError('Reporting of spans failed, upgrade your client library');
            function_exists('dd_tracer_circuit_breaker_register_error') && dd_tracer_circuit_breaker_register_error();
            return;
        }
        if ($statusCode !== 200) {
            self::logError('Reporting of spans failed, status code {code}: {response}', array('code' => $statusCode, 'response' => $response));
            function_exists('dd_tracer_circuit_breaker_register_error') && dd_tracer_circuit_breaker_register_error();
            return;
        }
        function_exists('dd_tracer_circuit_breaker_register_success') && dd_tracer_circuit_breaker_register_success();
        self::logDebug('Traces successfully sent to the agent');
    }
    private function isPrioritySamplingUsed()
    {
        $tracer = GlobalTracer::get();
        return \ddtrace_config_priority_sampling_enabled() && $tracer->getPrioritySampling() !== PrioritySampling::UNKNOWN;
    }
}
}
namespace DDTrace\Transport {
use DDTrace\Contracts\Tracer as TracerInterface;
use DDTrace\Transport;
final class Internal implements Transport
{
    public function send(TracerInterface $tracer)
    {
        if (!\dd_trace_env_config('DD_TRACE_GENERATE_ROOT_SPAN')) {
            \DDTrace\flush();
        }
    }
    public function setHeader($key, $value)
    {
    }
}
}
namespace DDTrace {
use DDTrace\Contracts\Tracer as TracerInterface;
interface Encoder
{
    public function encodeTraces(TracerInterface $tracer);
    public function getContentType();
}
}
namespace DDTrace\Processing {
use DDTrace\Data\Span as DataSpan;
use DDTrace\Tag;
final class TraceAnalyticsProcessor
{
    public static function normalizeAnalyticsValue(&$metrics, $value)
    {
        if (true === $value) {
            $metrics[Tag::ANALYTICS_KEY] = 1.0;
        } elseif (false === $value) {
            unset($metrics[Tag::ANALYTICS_KEY]);
        } elseif (is_numeric($value) && 0 <= $value && $value <= 1) {
            $metrics[Tag::ANALYTICS_KEY] = (double) $value;
        }
    }
}
}
namespace DDTrace {
use DDTrace\Contracts\Scope as ScopeInterface;
use DDTrace\Contracts\Span as SpanInterface;
final class Scope implements ScopeInterface
{
    private $span;
    private $scopeManager;
    private $finishSpanOnClose;
    public function __construct(ScopeManager $scopeManager, SpanInterface $span, $finishSpanOnClose)
    {
        $this->scopeManager = $scopeManager;
        $this->span = $span;
        $this->finishSpanOnClose = $finishSpanOnClose;
    }
    public function close()
    {
        if ($this->finishSpanOnClose) {
            $this->span->finish();
        }
        $this->scopeManager->deactivate($this);
    }
    public function getSpan()
    {
        return $this->span;
    }
}
}
namespace DDTrace {
use DDTrace\Contracts\SpanContext as SpanContextInterface;
interface Propagator
{
    const DEFAULT_BAGGAGE_HEADER_PREFIX = 'ot-baggage-';
    const DEFAULT_TRACE_ID_HEADER = 'x-datadog-trace-id';
    const DEFAULT_PARENT_ID_HEADER = 'x-datadog-parent-id';
    const DEFAULT_SAMPLING_PRIORITY_HEADER = 'x-datadog-sampling-priority';
    const DEFAULT_ORIGIN_HEADER = 'x-datadog-origin';
    public function inject(SpanContextInterface $spanContext, &$carrier);
    public function extract($carrier);
}
}
namespace DDTrace\Encoders {
use DDTrace\Span;
use DDTrace\Data\Span as DataSpan;
use DDTrace\Log\LoggingTrait;
use DDTrace\Tag;
final class SpanEncoder
{
    use LoggingTrait;
    public static function encode(DataSpan $span)
    {
        self::logSpanDetailsIfDebug($span);
        $arraySpan = array('trace_id' => $span->context->traceId, 'span_id' => $span->context->spanId, 'name' => $span->operationName, 'resource' => $span->resource, 'service' => $span->service, 'start' => (int) ($span->startTime . '000'), 'error' => $span->hasError ? 1 : 0);
        if ($span->type !== null) {
            $arraySpan['type'] = $span->type;
        }
        if ($span->duration !== null) {
            $arraySpan['duration'] = (int) ($span->duration . '000');
        }
        if ($span->context->parentId !== null) {
            $arraySpan['parent_id'] = $span->context->parentId;
        }
        $tags = $span->tags;
        if (!empty($span->context->origin) && $span->context->isHostRoot()) {
            $tags[Tag::ORIGIN] = $span->context->origin;
        }
        if (!empty($tags)) {
            $arraySpan['meta'] = $tags;
        }
        $metrics = array();
        foreach ($span->metrics as $metricName => $metricValue) {
            $metrics[$metricName] = $metricValue;
        }
        if ($span->context->isHostRoot()) {
            if (\dd_trace_env_config('DD_TRACE_MEASURE_COMPILE_TIME')) {
                $metrics['php.compilation.total_time_ms'] = (double) dd_trace_compile_time_microseconds() / 1000;
            }
        }
        if (!empty($metrics)) {
            $arraySpan['metrics'] = $metrics;
        }
        return $arraySpan;
    }
    private static function logSpanDetailsIfDebug(Span $span)
    {
        if (!self::isLogDebugActive()) {
            return;
        }
        $lengths = array();
        foreach ($span->getAllTags() as $tagName => $tagValue) {
            $lengths[] = "{$tagName}:" . strlen($tagValue);
        }
        self::logDebug('Encoding span \'{id}\' op: \'{operation}\' serv: \'{service}\' res: \'{resource}\' type \'{type}\'', array('id' => $span->getSpanId(), 'operation' => $span->getOperationName(), 'service' => $span->getService(), 'resource' => $span->getResource(), 'type' => $span->getType()));
        self::logDebug('Tags for span {id} \'tag:chars_count\' are: {lengths}', array('id' => $span->getSpanId(), 'lengths' => implode(',', $lengths)));
    }
}
}
namespace DDTrace\Encoders {
use DDTrace\Contracts\Tracer;
use DDTrace\Encoder;
use DDTrace\Log\LoggingTrait;
final class MessagePack implements Encoder
{
    use LoggingTrait;
    public function encodeTraces(Tracer $tracer)
    {
        $messagePack = \dd_trace_serialize_msgpack($tracer->getTracesAsArray());
        if (false === $messagePack) {
            self::logDebug('Failed to MessagePack-encode trace');
            return '';
        }
        return $messagePack;
    }
    public function getContentType()
    {
        return 'application/msgpack';
    }
}
}
namespace DDTrace\Propagators {
use DDTrace\Log\LoggingTrait;
use DDTrace\Propagator;
use DDTrace\Sampling\PrioritySampling;
use DDTrace\Contracts\SpanContext as SpanContextInterface;
use DDTrace\Contracts\Tracer;
use DDTrace\SpanContext;
final class TextMap implements Propagator
{
    use LoggingTrait;
    private $tracer;
    public function __construct(Tracer $tracer)
    {
        $this->tracer = $tracer;
    }
    public function inject(SpanContextInterface $spanContext, &$carrier)
    {
        $carrier[Propagator::DEFAULT_TRACE_ID_HEADER] = $spanContext->getTraceId();
        $carrier[Propagator::DEFAULT_PARENT_ID_HEADER] = $spanContext->getSpanId();
        foreach ($spanContext as $key => $value) {
            $carrier[Propagator::DEFAULT_BAGGAGE_HEADER_PREFIX . $key] = $value;
        }
        $prioritySampling = $this->tracer->getPrioritySampling();
        if (PrioritySampling::UNKNOWN !== $prioritySampling) {
            $carrier[Propagator::DEFAULT_SAMPLING_PRIORITY_HEADER] = $prioritySampling;
        }
        if (!empty($spanContext->origin)) {
            $carrier[Propagator::DEFAULT_ORIGIN_HEADER] = $spanContext->origin;
        }
    }
    public function extract($carrier)
    {
        $traceId = '';
        $spanId = '';
        $baggageItems = array();
        foreach ($carrier as $key => $value) {
            if ($key === Propagator::DEFAULT_TRACE_ID_HEADER) {
                $traceId = $this->extractStringOrFirstArrayElement($value);
            } elseif ($key === Propagator::DEFAULT_PARENT_ID_HEADER) {
                $spanId = $this->extractStringOrFirstArrayElement($value);
            } elseif (strpos($key, Propagator::DEFAULT_BAGGAGE_HEADER_PREFIX) === 0) {
                $baggageItems[substr($key, strlen(Propagator::DEFAULT_BAGGAGE_HEADER_PREFIX))] = $value;
            }
        }
        if (preg_match('/^\\d+$/', $traceId) !== 1 || preg_match('/^\\d+$/', $spanId) !== 1) {
            return null;
        }
        $spanContext = new SpanContext($traceId, $spanId ?: '', null, $baggageItems, true);
        $this->extractPrioritySampling($spanContext, $carrier);
        $this->extractOrigin($spanContext, $carrier);
        return $spanContext;
    }
    private function extractStringOrFirstArrayElement($value)
    {
        $singleValue = $this->extractScalarOrFirstArrayElement($value);
        if (is_string($singleValue)) {
            return $singleValue;
        }
        return null;
    }
    private function extractScalarOrFirstArrayElement($value)
    {
        if (is_array($value) && count($value) > 0) {
            return $value[0];
        } elseif (is_scalar($value)) {
            return $value;
        }
        return null;
    }
    private function extractPrioritySampling(SpanContextInterface $spanContext, $carrier)
    {
        if (isset($carrier[Propagator::DEFAULT_SAMPLING_PRIORITY_HEADER])) {
            $rawValue = $this->extractScalarOrFirstArrayElement($carrier[Propagator::DEFAULT_SAMPLING_PRIORITY_HEADER]);
            $spanContext->setPropagatedPrioritySampling(PrioritySampling::parse($rawValue));
        }
    }
    private function extractOrigin(SpanContextInterface $spanContext, $carrier)
    {
        if (property_exists($spanContext, 'origin') && isset($carrier[Propagator::DEFAULT_ORIGIN_HEADER])) {
            $spanContext->origin = $this->extractStringOrFirstArrayElement($carrier[Propagator::DEFAULT_ORIGIN_HEADER]);
        }
    }
}
}
namespace DDTrace\Http {
class Request
{
    public static function getHeaders(array $server = array())
    {
        $headers = array();
        $server = $server ?: $_SERVER;
        foreach ($server as $key => $value) {
            if (strpos($key, 'HTTP_') !== 0) {
                continue;
            }
            $key = substr($key, 5);
            $key = str_replace(' ', '-', strtolower(str_replace('_', ' ', $key)));
            $headers[$key] = $value;
        }
        return $headers;
    }
}
}
namespace DDTrace {
use DDTrace\Contracts\Scope as ScopeInterface;
use DDTrace\Contracts\ScopeManager as ScopeManagerInterface;
use DDTrace\Contracts\Span as SpanInterface;
use DDTrace\Util\ObjectKVStore;
final class ScopeManager implements ScopeManagerInterface
{
    private $scopes = array();
    private $hostRootScopes = array();
    private $rootContext;
    public function __construct(SpanContext $rootContext = null)
    {
        $this->rootContext = $rootContext;
    }
    public function activate(SpanInterface $span, $finishSpanOnClose = self::DEFAULT_FINISH_SPAN_ON_CLOSE)
    {
        $scope = new Scope($this, $span, $finishSpanOnClose);
        if ($span instanceof Span && ObjectKVStore::get($span->internalSpan, 'ddtrace_scope_activated') !== null) {
            ObjectKVStore::put($span->internalSpan, 'ddtrace_scope_activated', true);
        }
        $this->scopes[count($this->scopes)] = $scope;
        if ($span->getContext()->isHostRoot()) {
            $this->hostRootScopes[] = $scope;
        }
        return $scope;
    }
    public function getActive()
    {
        $this->reconcileInternalAndUserland();
        for ($i = count($this->scopes) - 1; $i >= 0; --$i) {
            $scope = $this->scopes[$i];
            $span = $scope->getSpan();
            if (!$span instanceof Span || ObjectKVStore::get($span->internalSpan, 'ddtrace_scope_activated', true)) {
                return $scope;
            }
        }
        return null;
    }
    public function deactivate(Scope $scope)
    {
        $i = array_search($scope, $this->scopes, true);
        if (false === $i) {
            return;
        }
        array_splice($this->scopes, $i, 1);
        $span = $scope->getSpan();
        if ($span instanceof Span && ObjectKVStore::get($span->internalSpan, 'ddtrace_scope_activated') !== null) {
            ObjectKVStore::put($span->internalSpan, 'ddtrace_scope_activated', false);
        }
    }
    public function getPrimaryRoot()
    {
        $this->reconcileInternalAndUserland();
        return reset($this->scopes) ?: null;
    }
    public function getTopScope()
    {
        return $this->reconcileInternalAndUserland();
    }
    private function reconcileInternalAndUserland()
    {
        $topScope = null;
        for ($i = count($this->scopes) - 1; $i >= 0; --$i) {
            $topScope = $this->scopes[$i];
            if ($topScope->getSpan()->isFinished()) {
                unset($this->scopes[$i]);
            } else {
                break;
            }
        }
        if (empty($this->scopes)) {
            if ($internalRootSpan = root_span()) {
                $traceId = trace_id();
                if ($this->rootContext) {
                    $parentId = $this->rootContext->spanId;
                } else {
                    $parentId = null;
                }
                $context = new SpanContext($traceId, $internalRootSpan->id, $parentId, array());
                $context->parentContext = $this->rootContext;
                $topScope = $this->scopes[0] = new Scope($this, new Span($internalRootSpan, $context), false);
            } else {
                return null;
            }
        }
        $currentSpanId = $topScope->getSpan()->getSpanId();
        $newScopes = array();
        for ($span = active_span(); $span && $span->id != $currentSpanId; $span = $span->parent) {
            $scope = new Scope($this, new Span($span, new SpanContext(trace_id(), $span->id, $span->parent ? $span->parent->id : null)), true);
            $newScopes[] = $scope;
        }
        foreach (array_reverse($newScopes) as $scope) {
            $scope->getSpan()->getContext()->parentContext = end($this->scopes)->getSpan()->getContext();
            $this->scopes[count($this->scopes)] = $scope;
        }
        return $newScopes ? $newScopes[0] : $topScope;
    }
    public function close()
    {
        foreach ($this->hostRootScopes as $scope) {
            $scope->close();
        }
    }
}
}
namespace DDTrace {
use DDTrace\Contracts\Span as SpanInterface;
use DDTrace\Contracts\SpanContext as SpanContextInterface;
use DDTrace\Contracts\Tracer as TracerInterface;
use DDTrace\Exceptions\UnsupportedFormat;
use DDTrace\Log\LoggingTrait;
use DDTrace\Propagators\Noop as NoopPropagator;
use DDTrace\Propagators\TextMap;
use DDTrace\Transport\Internal;
use DDTrace\Transport\Noop;
use DDTrace\Transport\Noop as NoopTransport;
use DDTrace\Util\ObjectKVStore;
final class Tracer implements TracerInterface
{
    use LoggingTrait;
    const VERSION = '0.91.2';
    private $traces = array();
    private $transport;
    private $propagators;
    private $config = array('service_name' => PHP_SAPI, 'enabled' => true, 'global_tags' => array());
    private $scopeManager;
    private $serviceVersion;
    private $environment;
    private $rootContext;
    public function __construct(Transport $transport = null, array $propagators = null, array $config = array())
    {
        $this->transport = $transport ?: new Internal();
        $textMapPropagator = new TextMap($this);
        $this->propagators = $propagators ?: array(Format::TEXT_MAP => $textMapPropagator, Format::HTTP_HEADERS => $textMapPropagator);
        $this->config = array_merge($this->config, $config);
        foreach ($this->config['global_tags'] as $key => $val) {
            add_global_tag($key, $val);
        }
        $this->serviceVersion = \ddtrace_config_service_version();
        $this->environment = \ddtrace_config_env();
        $context = current_context();
        if (isset($context['distributed_tracing_parent_id'])) {
            $parentId = $context['distributed_tracing_parent_id'];
            $this->rootContext = new SpanContext($context['trace_id'], $parentId, null, array(), true);
            if (isset($context['distributed_tracing_origin'])) {
                $this->rootContext->origin = $context['distributed_tracing_origin'];
            }
        }
        $this->scopeManager = new ScopeManager($this->rootContext);
    }
    public function limited()
    {
        return dd_trace_tracer_is_limited();
    }
    public function reset()
    {
        if (_ddtrace_config_bool(ini_get('datadog.trace.enabled'), false)) {
            ini_set('datadog.trace.enabled', 0);
            ini_set('datadog.trace.enabled', 1);
        }
        $this->scopeManager = new ScopeManager($this->rootContext);
    }
    public static function noop()
    {
        return new self(new NoopTransport(), array(Format::BINARY => new NoopPropagator(), Format::TEXT_MAP => new NoopPropagator(), Format::HTTP_HEADERS => new NoopPropagator()), array('enabled' => false));
    }
    public function startSpan($operationName, $options = array())
    {
        if (!$this->config['enabled'] || !\ddtrace_config_trace_enabled()) {
            return NoopSpan::create();
        }
        if (!$options instanceof StartSpanOptions) {
            $options = StartSpanOptions::create($options);
        }
        $reference = $this->findParent($options->getReferences());
        $roundedStartTime = $options->getStartTime() ? ($options->getStartTime() + 0.2) / 1000000 : 0;
        if ($reference === null) {
            $context = SpanContext::createAsRoot(array(), $roundedStartTime);
        } else {
            $context = SpanContext::createAsChild($reference->getContext(), $roundedStartTime);
        }
        $resource = array_key_exists('resource', $this->config) ? (string) $this->config['resource'] : null;
        $service = $this->config['service_name'];
        $internalSpan = active_span();
        $internalSpan->name = (string) $operationName;
        $internalSpan->service = $service;
        $internalSpan->resource = $resource;
        if (!isset($internalSpan->metrics)) {
            $internalSpan->metrics = array();
        }
        if (!isset($internalSpan->meta)) {
            $internalSpan->meta = array();
        }
        $span = new Span($internalSpan, $context);
        foreach ($options->getTags() as $key => $val) {
            $span->setTag($key, $val);
        }
        ObjectKVStore::put($internalSpan, 'ddtrace_scope_activated', false);
        return $span;
    }
    public function startRootSpan($operationName, $options = array())
    {
        $rootScope = $this->startActiveSpan($operationName, $options);
        $this->setPrioritySamplingFromSpan($rootScope->getSpan());
        return $rootScope;
    }
    public function getRootScope()
    {
        return $this->scopeManager->getPrimaryRoot();
    }
    public function startActiveSpan($operationName, $options = array())
    {
        if (!$options instanceof StartSpanOptions) {
            $options = StartSpanOptions::create($options);
        }
        if (!root_span() && !$options->getReferences() && $this->rootContext) {
            $options = $options->withParent($this->rootContext);
        }
        $parentService = null;
        if (($activeScope = $this->scopeManager->getTopScope()) !== null) {
            $activeSpan = $activeScope->getSpan();
            $tags = $options->getTags();
            if (!array_key_exists(Tag::SERVICE_NAME, $tags)) {
                $parentService = $activeSpan->getService();
            }
            $options = $options->withParent($activeSpan);
        }
        $span = $this->startSpan($operationName, $options);
        if ($parentService !== null) {
            $span->setTag(Tag::SERVICE_NAME, $parentService);
        }
        $shouldFinish = $options->shouldFinishSpanOnClose() && ($span->getParentId() != 0 || !\dd_trace_env_config('DD_TRACE_GENERATE_ROOT_SPAN'));
        return $this->scopeManager->activate($span, $shouldFinish);
    }
    private function findParent(array $references)
    {
        foreach ($references as $reference) {
            if ($reference->isType(Reference::CHILD_OF)) {
                return $reference;
            }
        }
        return null;
    }
    public function inject(SpanContextInterface $spanContext, $format, &$carrier)
    {
        if (!array_key_exists($format, $this->propagators)) {
            throw UnsupportedFormat::forFormat($format);
        }
        $this->propagators[$format]->inject($spanContext, $carrier);
    }
    public function extract($format, $carrier)
    {
        if (array_key_exists($format, $this->propagators)) {
            return $this->propagators[$format]->extract($carrier);
        }
        throw UnsupportedFormat::forFormat($format);
    }
    public function flush()
    {
        if (!$this->config['enabled']) {
            return;
        }
        if ('cli' !== PHP_SAPI && \ddtrace_config_url_resource_name_enabled() && ($rootSpan = $this->getSafeRootSpan())) {
            $this->addUrlAsResourceNameToSpan($rootSpan);
        }
        if (self::isLogDebugActive()) {
            self::logDebug('Flushing {count} traces, {spanCount} spans', array('count' => $this->getTracesCount(), 'spanCount' => dd_trace_closed_spans_count()));
        }
        $this->transport->send($this);
    }
    public function getScopeManager()
    {
        return $this->scopeManager;
    }
    public function getActiveSpan()
    {
        if (null !== ($activeScope = $this->scopeManager->getActive())) {
            return $activeScope->getSpan();
        }
        return null;
    }
    public function getTracesAsArray()
    {
        $trace = \dd_trace_serialize_closed_spans();
        return $trace ? array($trace) : $trace;
    }
    public function addUrlAsResourceNameToSpan(Contracts\Span $span)
    {
        if (null !== $span->getResource()) {
            return;
        }
        if (!isset($_SERVER['REQUEST_METHOD'])) {
            return;
        }
        $resourceName = $_SERVER['REQUEST_METHOD'];
        if (isset($_SERVER['REQUEST_URI'])) {
            $resourceName .= ' ' . \DDTrace\Util\Normalizer::uriNormalizeIncomingPath($_SERVER['REQUEST_URI']);
        }
        $span->setTag(Tag::RESOURCE_NAME, $resourceName, true);
    }
    private function setPrioritySamplingFromSpan(SpanInterface $span)
    {
        if (!\ddtrace_config_priority_sampling_enabled()) {
            return;
        }
        if (!$span->getContext()->isHostRoot()) {
            return;
        }
        $prioritySampling = $span->getContext()->getPropagatedPrioritySampling();
        if (null !== $prioritySampling) {
            $this->setPrioritySampling($prioritySampling);
        }
    }
    public function setPrioritySampling($prioritySampling)
    {
        set_priority_sampling($prioritySampling);
        set_priority_sampling($prioritySampling, true);
    }
    public function getPrioritySampling()
    {
        return get_priority_sampling();
    }
    public function getSafeRootSpan()
    {
        $rootScope = $this->getRootScope();
        if (empty($rootScope)) {
            return null;
        }
        return $rootScope->getSpan();
    }
    public static function version()
    {
        return self::VERSION;
    }
    public function getTracesCount()
    {
        return count($this->traces);
    }
}
}
